#ifndef _JETSUTIL_H_
#define _JETSUTIL_H_

/*
  ---- Delphes ----
  A Fast Simulator for general purpose LHC detector
  S. Ovyn ~~~~ severine.ovyn@uclouvain.be
  
  Center for Particle Physics and Phenomenology (CP3)
  Universite Catholique de Louvain (UCL)
  Louvain-la-Neuve, Belgium
*/

/// \file SmearUtil.h
/// \brief RESOLution class, and some generic definitions


#include <vector>
#include "TLorentzVector.h"

#include "interface/SmearUtil.h"
#include "Utilities/ExRootAnalysis/interface/BlockClasses.h"
#include "Utilities/ExRootAnalysis/interface/TSimpleArray.h"
#include "Utilities/ExRootAnalysis/interface/ExRootTreeReader.h"
#include "Utilities/ExRootAnalysis/interface/ExRootTreeWriter.h"
#include "Utilities/ExRootAnalysis/interface/ExRootTreeBranch.h"



#include "Utilities/Fastjet/plugins/CDFCones/interface/PhysicsTower.hh"


#include "TRandom.h"

#include <iostream>
#include <sstream>
#include <fstream>
#include <iomanip>
#include <cstring>

#include "Utilities/Fastjet/include/fastjet/PseudoJet.hh"
#include "Utilities/Fastjet/include/fastjet/ClusterSequence.hh"

// get info on how fastjet was configured
#include "Utilities/Fastjet/include/fastjet/config.h"

// make sure we have what is needed
#ifdef ENABLE_PLUGIN_SISCONE
#  include "Utilities/Fastjet/plugins/SISCone/SISConePlugin.hh"
#endif
#ifdef ENABLE_PLUGIN_CDFCONES
#  include "Utilities/Fastjet/plugins/CDFCones/CDFMidPointPlugin.hh"
#  include "Utilities/Fastjet/plugins/CDFCones/CDFJetCluPlugin.hh"
#endif

using namespace std;

class JetsUtil
{
 public:
  /// Constructor
  JetsUtil(const string DetDatacard);


  fastjet::JetDefinition jet_def;
  fastjet::JetDefinition::Plugin * plugins;

  vector<fastjet::PseudoJet> inclusive_jets;
  vector<fastjet::PseudoJet> sorted_jets;
  vector<fastjet::PseudoJet> RunJets(const vector<fastjet::PseudoJet>&  input_particles);
  
  void RunJetBtagging(ExRootTreeWriter *treeWriter, ExRootTreeBranch *branchJet,const vector<fastjet::PseudoJet> & sorted_jets,const TSimpleArray<TRootGenParticle> & NFCentralQ);

  void RunTauJets(ExRootTreeWriter *treeWriter, ExRootTreeBranch *branchTauJet,const vector<fastjet::PseudoJet> & sorted_jets,const vector<PhysicsTower> & towers, const vector<TLorentzVector> & TrackCentral);

 private:

  RESOLution *DET;
};

#endif
